@@ -1,8 +1,9 @@
-__version__ = '2.0.0'
+__version__ = '5.0.1'
 
 from .retry import (
     retry,
     RetryPolicy,
     RetryPolicyStrategy,
-    AfterFailure
+    BeforeRetry,
+    RetryInfo
 )
