@@ -9,19 +9,60 @@ from typing import (
 
 import inspect
 import asyncio
+from datetime import datetime
+
+
+class RetryInfo:
+    __slots__ = (
+        'fails',
+        'exception',
+        'since'
+    )
+
+    fails: int
+    exception: Exception
+    since: datetime
+
+    def __init__(
+        self,
+        fails: int,
+        exception: Exception,
+        since: datetime
+    ) -> None:
+        self.fails = fails
+        self.exception = exception
+        self.since = since
+
+    def update(
+        self,
+        exception: Exception
+    ) -> 'RetryInfo':
+        """Create a new RetryInfo and update fails and exception
+
+        Why?
+            The object might be collected by user, so we need to create a new object every time it fails.
+        """
+
+        return RetryInfo(
+            self.fails + 1,
+            exception,
+            self.since
+        )
 
 
 RetryPolicyStrategy = Tuple[bool, Union[int, float]]
 
-RetryPolicy = Callable[[int], RetryPolicyStrategy]
-AfterFailure = Callable[[Exception, int], Optional[Awaitable]]
+RetryPolicy = Callable[[RetryInfo], RetryPolicyStrategy]
+BeforeRetry = Callable[[RetryInfo], Optional[Awaitable[None]]]
 
 ParamRetryPolicy = Union[RetryPolicy, str]
-ParamAfterFailure = Union[AfterFailure, str]
+ParamBeforeRetry = Union[BeforeRetry, str]
 
 TargetFunction = Callable[..., Awaitable]
+Exceptions = Tuple[Exception, ...]
+ExceptionsOrException = Union[Exceptions, Exception]
 
-T = TypeVar('T', RetryPolicy, AfterFailure)
+T = TypeVar('T', RetryPolicy, BeforeRetry)
 
 
 async def await_coro(coro):
@@ -32,26 +73,31 @@ async def await_coro(coro):
 
 
 async def perform(
-    fails: int,
     fn: TargetFunction,
     retry_policy: RetryPolicy,
-    before_retry: Optional[AfterFailure],
+    before_retry: Optional[BeforeRetry],
     *args,
     **kwargs
 ):
+    info = None
+
     while True:
         try:
             return await fn(*args, **kwargs)
         except Exception as e:
-            fails += 1
-            abandon, delay = retry_policy(fails)
+            if info is None:
+                info = RetryInfo(1, e, datetime.now())
+            else:
+                info = info.update(e)
+
+            abandon, delay = retry_policy(info)
 
             if abandon:
                 raise e
 
             if before_retry is not None:
                 try:
-                    await await_coro(before_retry(e, fails))
+                    await await_coro(before_retry(info))
                 except Exception as e:
                     raise RuntimeError(
                         f'[aioretry] before_retry failed, reason: {e}'
@@ -72,7 +118,7 @@ def get_method(
 
     if len(args) == 0:
         raise RuntimeError(
-            f'[aioretry] decorator should be used for instance method if {name} as a str `"{target}"` '
+            f'[aioretry] decorator should be used for instance method if {name} as a str `"{target}"`'
         )
 
     self = args[0]
@@ -82,12 +128,26 @@ def get_method(
 
 def retry(
     retry_policy: ParamRetryPolicy,
-    before_retry: Optional[ParamAfterFailure] = None
+    before_retry: Optional[ParamBeforeRetry] = None
 ) -> Callable[[TargetFunction], TargetFunction]:
+    """Creates a decorator function
+
+    Args:
+        retry_policy (RetryPolicy, str): the retry policy
+        before_retry (BeforeRetry, str, None): the function to be called after each failure of fn and before the corresponding retry.
+
+    Returns:
+        A wrapped function which accepts the same arguments as fn and returns an Awaitable
+
+    Usage::
+        @retry(retry_policy)
+        async def coro_func():
+            ...
+    """
+
     def wrapper(fn: TargetFunction) -> TargetFunction:
         async def wrapped(*args, **kwargs):
             return await perform(
-                0,
                 fn,
                 get_method(
                     retry_policy,
